% This m-file calculates the multi-polar DEP force on a particle in 
% arbitrary geometries under a range of conditions.
%
% Uses:
%   CM = cmfactor('bead',eps,sig,f,nmax)                     -  multipole CM
%   obj = defineobj(obj)                                     -  define object
%   [E,Fdepn,Fdep] = depforce3(x,y,z,Ex,Ey,Ez,CM,nmax,R,eps) - DEP force
%
% Variables:
%	E(x,y,z,p,q,r) 	 - E(x,y,z) of order p,q, and r. 
%			      E(x,y,z,2,1,1) = Ex(x,y,z)
%			      E(x,y,z,1,2,1) = Ey(x,y,z)
%	Fdepn(x,y,z,1|2|3,n) - n-th order dep force expression where 
%			    Fdepn(x,y,z,1,n) is n-th order Fx(x,y,z)
%			    Fdepn(x,y,z,2,n) is n-th order Fy(x,y,z)
%	Fdep(x,y,z,1|2|3)- total dep force expression in x (1), 
%			    y (2) or z (3)


% Universal Constants
g = 9.81;			    % gravity (m/s^2);
eps0 = 8.85e-12;		% permittivity of free space

% initialize E matrix to reserve memory
E = zeros(size(ExI,1),size(ExI,2),size(ExI,3),nmax+1,nmax+1,nmax+1);
clear E

% Generate physical space
if ~exist('x')              % if we haven't done it before...
    x = XI;
    y = YI;
    z = ZI;
    xp = XI*1e6;
    yp = YI*1e6;
    zp = ZI*1e6;
elseif size(x)~=size(ExI)   % or if it is different in this simulation....
    x = XI;
    y = YI;;
    z = ZI;
    xp = XI*1e6;
    yp = YI*1e6;
    zp = ZI*1e6;
end


% Generate E-field
fprintf('Scaling E-field...');
Ex = ExI * V/Vsim;        % Scale to appropriate voltage
Ey = EyI * V/Vsim;
Ez = EzI * V/Vsim;

% Find DEP forces and potentials
fprintf('Computing DEP forces\n');
obj=defineobj(obj);                         % fill in any missing object parameters
CM = cmfactor(obj,media,f,nmax);			% multi-order CM factor
[E,Fdepn,Fdep] = depforce3(x,y,z,Ex,Ey,Ez,CM,nmax,obj.R,media.eps);  % calculate DEP force
