% CMFACTOR computes the n-th order clausius-mossotti factor for the DEP force 
% expression for various particles.
%
% CM = CMFACTOR(OBJECT,MEDIA,FREQ,NMAX) is the traditional form and takes
% as its input:
%       OBJECT	- a structure with various subentries.  The following
%         fields are needed for CMFACTOR, depending on the OBJECT type.
%         OBJECT.TYPE='bead' is a uniform sphere and must include .EPS, .SIG
%                    ='cell' is a membrane-covered sphere and must include 
%                         .EPSCYTO,.SIGCYTO,.R,.GM,.CM,.H
%                    ='virus' is a membrane-covered sphere with significant
%                         surface conduction, and must include .EPSCYTO,
%                         .SIGCYTO,.R,.GM,.CM,.H,.SC
%                    ='bacteria' is a membrane-covered particle (which does 
%                         not have to be a sphere) including a
%                         cell wall, and must include .EPSCYTO,.SIGCYTO,.R,
%                         .GM,.CM,.H,.WALLEPS,.WALLSIG,.A,.B
%       MEDIA   - a structure including .sig (conductivity, S/m) and .eps
%                 (rel. permittivity)
%		FREQ	- frequency (Hz) vector over which to calc. CM (vector or scalar)
%		NMAX	- maximum order of CM requested
%
%   Only the 1st-order CM factor can be calculated for the 'bacteria'
%
%  You can pass a null or a vector for media, such as
%  CMFACTOR(OBJECT,[],...) or CMFACTOR(OBJECT,0.01,...) or
%  CMFACTOR(OBJECT,[80 0.01],...) which will deal with the inputs correctly
%
%  Outputs:	CM	- (freq x nmax) matrix giving COMPLEX CM factor

function CM = cmfactor(object,media,varargin)

if nargin<3
    fprintf('Something is missing--three input arguments needed');return
elseif nargin==3
    nmax=1;     %specify nmax as it is missing
else
    nmax=varargin{2};
end

% Check to make sure inputs are in order
if ~isfield(object,'sigcyto')
  switch object.type;case {'bacteria','virus','cell'};error('Cytoplasmic conductivity needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'epscyto')
  switch object.type;case {'bacteria','virus','cell'};error('Cytoplasmic permittivity needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'R')
  switch object.type;case {'virus','cell'};error('Radius needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'a')     
  switch object.type;case {'bacteria'};error('Long axis needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'b')     
  switch object.type;case {'bacteria'};error('Short axes needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'gm')      
  switch object.type;case {'bacteria','virus','cell'};error('gm needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'cm')
  switch object.type;case {'bacteria','virus','cell'};error('cm needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'h')
  switch object.type;case {'bacteria','virus','cell'};error('h needed, define or run DEFINEOBJ');
end;end
if ~isfield(object,'sc')
  switch object.type;case {'virus'};error('surface conductance sc needed, define or run DEFINEOBJ');
end;end
if or(~isfield(object,'walleps'),~isfield(object,'wallsig'))
  switch object.type;case {'bacteria'};error('cell wall parameter needed, define or run DEFINEOBJ');
end;end
if or(~isfield(object,'eps'),~isfield(object,'sig'))
  switch object.type;case {'bead'};error('bead parameter needed, define or run DEFINEOBJ');
end;end
if nmax>1 & strcmp(object.type,'bacteria')
    error('Can only do 1st-order CM factor for these particles');
end
      
%fill in media fields as necessary
if ~isstruct(media) & length(media)~=0          %if we pass in a 1 or 2-element vector
    if length(media)==1
        media.sig=media;                        %1-element means we are specifying cond
        media.eps=80;
    else
        mm=media;
        media.sig=mm(2);                     %2-element we just put stuff in the right place
        media.eps=mm(1);
    end
end
%if we pass nothing or part of a structure for media, then fill in the rest
if ~isfield(media,'sig');media.sig=1.0;end
if ~isfield(media,'eps');media.eps=80;end


% Physical constants
eps0 = 8.85e-12;		% permittivity of free space

% Compute input params
n = 1:nmax;                 % # of orders to compute
f = varargin{1}';			% frequency (vector or scalar)
w = 2*pi*f;                 % radians
nvec = ones(nmax,1);		% these two vectors serve in cell calcs
fvec = ones(1,length(f));	% to transform vectors in matrices

% BEAD
if strcmp(object.type,'bead')
      eps= [media.eps object.eps];
      sig= [media.sig object.sig];
      e = eps'*eps0*fvec - j * sig' * w.^-1';
      CM = nvec*(e(2,:) - e(1,:)) ./( (n+1)'*e(1,:) + n'*e(2,:));
      CM = CM';

% MAMMALIAN CELL OR VIRUS
elseif strcmp(object.type,'cell') | strcmp(object.type,'virus')
      eps= [media.eps object.epscyto];
      sig= [media.sig object.sigcyto];
      R = object.R;             % cell radius (m)
      cm = object.cm*1e4;		% membrane capacitance F/m^2
      gm = object.gm*1e4;		% membrane conductance S/m^2
      if strcmp(object.type,'virus')
          gm = object.gm*1e4 + 2*object.sc/R^2;
      end
      h = object.h;			% membrane thickness (m)
      a = (h+R)/R;			% ratio of outer/inner radii

      e1 = ( eps(1)*eps0 + sig(1)./(j*w) )';	% compute freq. dependent
      e3 = ( eps(2)*eps0 + sig(2)./(j*w) )';	% complex
      e2 = ( cm*h + gm*h./(j*w) )';		        % permittivities

      n = n';				% transpose n to make calcs cleaner 
      Kn = (nvec*(e3 - e2)) ./ (n*e3 + (n+1)*e2);
      num = a.^(2*n+1)*fvec + ( (n+1)*fvec ) .* Kn;
      den = a.^(2*n+1)*fvec - (   n  *fvec ) .* Kn;
      e2pn = (nvec * e2) .* (num./den);
      CM = (e2pn - nvec*e1) ./ ( (n+1)*e1 + n*fvec.*e2pn);
      CM = CM';

% BACTERIA
elseif strcmp(object.type,'bacteria')
  % can only do first order CM factor
  eps= [media.eps object.epscyto];
  sig= [media.sig object.sigcyto];
  R = object.R;			% cell radius (m)
  cm = object.cm*1e4;		% membrane capacitance F/m^2
  gm = object.gm*1e4;		% membrane conductance S/m^2
  h = object.h;			% membrane thickness (m)
  a = (h+R)/R;			% ratio of outer/inner radii

  e1 = ( eps(1)*eps0 + sig(1)./(j*w) )';	% media
  e2 = ( eps(2)*eps0 + sig(2)./(j*w) )';	% cytoplasm
  e4 = ( object.walleps*eps0 + object.wallsig./(j*w) )';	% cell wall
  e3p = cm*R*e2 ./ (cm*R + e2);            % cell membrane
  e2p = e4 .* ( a^3 + 2*(e3p-e4)./(e3p+2*e4) ) ./ ( a^3 - (e3p-e4)./(e3p+2*e4) );
  CM = (e2p - e1) ./ ( e2p + 2*e1);
end