% DEFINEOBJ creates a structure containing the necessary data for
% different particle types.
%
% NEWOBJ=DEFINEOBJ(OBJ) takes as its input a structure OBJ and outputs
% the structure NEWOBJ that fills in an unspecified values in OBJ from a
% database in the program.  It can handle four different particle types:
% bead, cell, virus, and bacteria.  
% 
% The input structure OBJ must contain at least the .TYPE field specifying
% the particle type.  Other fields are type-specific:
%       OBJ	- a structure with various subentries
%          .TYPE can be 
%             'bead' -- uniform sphere
%                    includes
%                       OBJECT.EPS  -- relative permittivity
%                       OBJECT.SIG  -- conductivity (S/m)
%                       OBJECT.R    -- radius (m)
%                       OBJECT.RHO  -- density
%             'cell' -- mammalian cell or membrane-covered sphere
%                    includes
%                       OBJECT.EPSCYTO  -- cytoplasmic relative permittivity
%                       OBJECT.SIGCYTO  -- cytoplasmic conductivity (S/m)
%                       OBJECT.R   -- radius (m)
%                       OBJECT.RHO  -- density
%                       OBJECT.GM  -- membrane conductance (S/cm2)
%                       OBJECT.CM  -- membrane capacitance (F/cm2)
%                       OBJECT.H   -- membrane thickness
%             'virus' -- membrane-covered sphere w/ significant surface 
%                         conduction, as would occur for nm-sized particles
%                    includes all subentries as 'cell' plus
%                       OBJECT.SC  -- surface conductance (S)
%             'bacteria' -- sphere with membrane and cell wall, models
%                        bacteria and round bacteria.  Only 1st-order CM
%                        factor can be computer
%                    includes all subentries as 'cell' plus
%                       OBJECT.WALLEPS   -- wall relative permittivity
%                       OBJECT.WALLSIG   -- wall conductivity (S/m)
%   
%       --For 'bead', assume Bangs Labs polystyrene beads with permittivity
%         2.5 and conductivity 2e-4 S/m, R=5 um, rho=1.062 g/cm2
%       --For 'cell' uses values for HL-60s from Gascoyne, eps_cyto=75,
%         sig_cyto=0.75 S/m, cm=1.6e-6 F/cm2, gm=22e-3 S/cm2, R=5 um, h=1
%         nm, rho=1.071 g/cm2
%       --For 'virus' uses values for HSV-1 from Hughes et al. BBA 1425(1998)
%         p.119; eps_cyto=70, sig_cyto=5 mS/m, cm=9e-7 F/cm2, gm=0, R=50 nm, 
%         h=10 nm, surf cond.=3.5 nS
%       --For 'bacteria' uses values for E. coli from Suehiro et al., J Elect. 57(2003)
%         p. 157; eps_cyto=60, sig_cyto=0.1 S/m, cm=1.8e-6 F/cm2, gm=1 mS/m, R=1 um, 
%         h=20 nm, wall permittivity=60, wall conductivity 0.5 S/m
%
%  You can also pass a string as object, for instance DEFINEOBJ('BEAD')

function newobj=defineobj(object)

if ~isfield(object,'type');object.type=object;end       %if object was put in as a string
%fill in object fields as necessary
switch object.type
    case 'bead'
      if ~isfield(object,'sig');object.sig=2e-4;end
      if ~isfield(object,'eps');object.eps=2.5;end
      if ~isfield(object,'R');object.R=5e-6;end        % typical value
      if ~isfield(object,'rho');object.rho=1062;end
    case 'cell'
      if ~isfield(object,'sigcyto');object.sigcyto=0.75;end
      if ~isfield(object,'epscyto');object.epscyto=75;end
      if ~isfield(object,'R');object.R=5e-6;end
      if ~isfield(object,'gm');object.gm=22e-3;end
      if ~isfield(object,'cm');object.cm=1.6e-6;end
      if ~isfield(object,'h');object.h=1e-9;end
      if ~isfield(object,'rho');object.rho=1071;end      
    case 'virus'
      if ~isfield(object,'sigcyto');object.sigcyto=5e-3;end
      if ~isfield(object,'epscyto');object.epscyto=70;end
      if ~isfield(object,'R');object.R=50e-9;end
      if ~isfield(object,'gm');object.gm=0;end
      if ~isfield(object,'cm');object.cm=9e-7;end
      if ~isfield(object,'h');object.h=10e-9;end
      if ~isfield(object,'sc');object.sc=3.5e-9;end
      if ~isfield(object,'rho');object.rho=1071;end      % from 'cell'
    case 'bacteria'
      if ~isfield(object,'sigcyto');object.sigcyto=0.1;end
      if ~isfield(object,'epscyto');object.epscyto=60;end
      if ~isfield(object,'R');object.R=1e-6;end
      if ~isfield(object,'gm');object.gm=1e-3;end
      if ~isfield(object,'cm');object.cm=1.8e-6;end
      if ~isfield(object,'h');object.h=20e-9;end
      if ~isfield(object,'walleps');object.walleps=60;end      
      if ~isfield(object,'wallsig');object.wallsig=0.5;end      
      if ~isfield(object,'rho');object.rho=1071;end      % from 'cell'
  end

newobj=object;
