%   DEPFORCE3  Calculates the multipolar DEP force everywhere in space
% [E,FDEPN,FDEP] = DEPFORCE3(X,Y,Z,EX,EY,EZ,CM,NMAX,R,EPS)computes the n-th 
% order DEP force for a three-dimensional electric field geometry.  It has 
% a script for going through the gradient permutations efficiently.
%
% This algorithm should compute the correct force in both static
% and traveling wave configurations.  In order to do this it needs the
% complex CM factor and complex e-fields
%
%   Inputs:	
%       X,Y,Z	 - space variables (3D matrices)
%		EX,EY,EZ - E-field in x/y/z-dir (3D matrices, may be complex)
%		CM 	     - Complex Clausius Mossotti factor (1 x nmax)
%		NMAX 	 - max order to compute dep force (dipole, quadrupole,etc.)
%		R	     - radius of particle
%		EPS	     - relative permittivity of medium 
% 
%   Outputs:	E(x,y,z,p,q,r) 	- E(x,y,z) of order p and q and r.  Here p,
%				    q, and r refer to the number of derivatives of the 
%                   electric field taken in each direction, where 1 is a
%                   placeholder due to matlab's non-zero indexing scheme,
%                   2 is no derivative but specifies direction, 3 is the 
%                   1st derivative, etc.  
%                   For instance: 
%				      E(x,y,z,2,1,1) = Ex(x,y,z)
%				      E(x,y,z,1,2,1) = Ey(x,y,z)
%				      E(x,y,z,3,1,3) = d2/dz2 d/dx Ex(x,y,z)
%		FDEPN(x,y,z,1|2|3,n) - 5-D matrix containing nth-order DEP force, 
%                   where 
%				      F(x,y,z,1,n) is n-th order Fx(x,y,z)
%				      F(x,y,z,2,n) is n-th order Fy(x,y,z)
%				      F(x,y,z,3,n) is n-th order Fz(x,y,z)
%
%		FDEP(x,y,z,1|2|3)- 4-D matrix containing total DEP force in x (1), 
%                   y (2), or z (3)
%

function [E,F,Ftot] = depforce3(x,y,z,Ex,Ey,Ez,CM,nmax,R,eps)

%physical constants
eps0 = 8.85e-12;			% permittivity of free space

% Calculate grid spacing dx,dy,dz
dx = abs(x(1,2,1)-x(1,1,1));	
dy = abs(y(2,1,1)-y(1,1,1));
dz = abs(z(1,1,2)-z(1,1,1));

% Set Ex, Ey, and Ez to respective areas of 6-dim vectors
fprintf('Initializing E matrix...');
E = zeros(size(Ex,1),size(Ex,2),size(Ex,3),nmax+1,nmax+1,nmax+1);
E(:,:,:,2,1,1) = Ex;
E(:,:,:,1,2,1) = Ey;
E(:,:,:,1,1,2) = Ez;

% Calculate complex freq. dependent term involving CM factor
n = 1:nmax;
for in=1:nmax
  nfact2(in) = prod((2*n(in)-1):-2:1);		% nfact2 is (2n-1)*(2n-3)...*1
  nfact(in) = prod(n(in):-1:1);			    % nfact is n*(n-1)*...*1
end
% This prefactor is a combination of Eqn 22, 23 and 25 in Jones and Washizu
% J Elec. 37:121, 1996.
F0 = 2*pi*eps*eps0*R.^(2*n+1).*n.*CM ./ (nfact2 .* nfact);

% Compute gradients of electric fields
fprintf('Computing field gradients of order - ');
for in=2:nmax+1
  fprintf(' %d ',in-1);
  for p=1:in
    for q=1:(in-p+1)
      r = in - (q + p) + 2;
      [E(:,:,:,p+1,q,r),E(:,:,:,p,q+1,r),E(:,:,:,p,q,r+1)] = gradient(E(:,:,:,p,q,...
									r),dx,dy,dz);
    end
  end
end

% Compute forces F(x,y,z,1 or 2 or 3,n)
% Here we are implementing an E.grad(E) version of the DEP force, as in
% eqn 25 in Jones and Washizu, J Elec. 37:121, 1996.
fprintf('\nInitializing Fdep and computing forces of order - ');
F = zeros(size(Ex,1),size(Ex,2),size(Ex,3),3,nmax);
Ftot = zeros(size(Ex,1),size(Ex,2),size(Ex,3),3);
for in=1:nmax
  fprintf(' %d',in);
  [ireg,idx,idy,idz] = findeterms(in);  % pick correct indices to add up
  for in1=1:size(ireg,2);
      F(:,:,:,1,in) = F(:,:,:,1,in) + real(F0(in)*E(:,:,:,ireg(1,in1),ireg(2,in1),...
          ireg(3,in1)).*conj(E(:,:,:,idx(1,in1),idx(2,in1),idx(3,in1))));
      F(:,:,:,2,in) = F(:,:,:,2,in) + real(F0(in)*E(:,:,:,ireg(1,in1),ireg(2,in1),...
          ireg(3,in1)).*conj(E(:,:,:,idy(1,in1),idy(2,in1),idy(3,in1))));
      F(:,:,:,3,in) = F(:,:,:,3,in) + real(F0(in)*E(:,:,:,ireg(1,in1),ireg(2,in1),...
          ireg(3,in1)).*conj(E(:,:,:,idz(1,in1),idz(2,in1),idz(3,in1))));
  end
end
Ftot = sum(F,5);
fprintf('\n');



% FUNCTION [i1,ix,iy,iz]=findeterms(nm)
%
% Generates the list of appropriate e-field terms to add up to create force
% representation of a given DEP order nm.  First creates a nm-dimensional
% matrix where each dimension has length 3.  The terms in the e-field sum
% are given by the x,y,z indices of each of the elements in that matrix.
% From those indices, we count the number of 1's, 2's, and 3's to determine
% the appropriate e-field term, where 1==x, 2==y, 3==z.  
% 
% For instance, nm=4 will generate a 3x3x3x3 matrix.  One of the elements
% will be (1,1,2,3).  That corresponds to E211 because there are two 1's,
% one 2, and one 3.
%
% Inputs:   nm          DEP order to calculate terms for
%
% Outputs:  i1          3 x 3^nm matrix where each column refers to an E-field
%                       term.  e.g., [4;2;1] = E421
%           ix,iy,iz    same as i1 but 1st/2nd/3rd row augmented by one to
%                       reflect taking derivative in dx, dy, or dz.
%
function [i1,ix,iy,iz ]  = findeterms(nm);


numel = 3^nm;             %number of E*E pairs to add
dims = 3*ones(1,nm);      %size of ndim matrix. 1x3, 3x3, 3x3x3, etc.

[I{1:nm}]=ind2sub(dims,1:numel);      %find indexes of all matrix elements

% pack the cell array I into a nmax x numel 2-D array of indexes
for in=1:nm
    II(in,:)=I{in};
end

indmat = zeros(3,numel);          %will hold counts of each index in x,y,z

% generate counts
for in=1:numel;
    indmat(1,in)=length(find(II(:,in)==1));
    indmat(2,in)=length(find(II(:,in)==2));
    indmat(3,in)=length(find(II(:,in)==3));
end

i1 = indmat+1;      % augment to implement matlab's non-zero matrix addressing scheme
ix = i1 + [ones(numel,1),zeros(numel,1),zeros(numel,1)]';
iy = i1 + [zeros(numel,1),ones(numel,1),zeros(numel,1)]';
iz = i1 + [zeros(numel,1),zeros(numel,1),ones(numel,1)]';