%   DRAGFORCE  Calculate the drag force on a sphere in parallel-plate flow
% FX = DRAGFORCE(R, Q, H, W, Z, TYP) calculates the drag force coefficient 
% FX as a function of the distance of the sphere center from the lower
% wall.  We find the drag by interpolating within the solutions for a
% stationary sphere in shear or parabolic flow.  We use the solution from 
% Goldman et al., Chem Eng Sci 1967(22)653 for shear flow and Ganatos et 
% al., J Fluid Mech 1980(99)755 for parabolic flow.
% 
% Inputs:	R       sphere radius (m)
%           Q       volumetric flowrate (m^3/s)
%           H       height of channel (m)
%           W       width of channel (m)
%           Z       height vector along which to calculate drag (m)
%           typ     pick flow profile and interpolation scheme
%               1: Shear     , linear
%               2: Poiseiulle, linear
%               3: Shear     , cubic
%               4: Poiseiulle, cubic
%               5: Shear     , spline
%               6: Poiseiulle, spline
%
% Outputs:	FX	vector of drag force vs. z in newtons
%
function Fx = dragforce(R, Q, h, w, z0,typ)

if nargin==5;typ=1;end			% set lin/shear as default

if Q==0;Fx=0*z;return;end         % for null case, return to avoid matlab warnings

% Fluid properties evaluated at T=295K (22C)
nu = 0.97e-6;	% nu is the kinematic viscosity of the fluid, water (m^2/s)
mu = 9.68e-4;	% mu is the viscosity of the fluid, water (kg/ms)

% Variables
Ac = h * w;                 % Cross-sectional area of the channel (m^2)
z_n = z0 / h;               % normalized distance from substrate
z_R = z0 / R;               % dist normalized to sphere radius
z_R = round(z_R/1e-3)*1e-3;	% and then rounded

% Flow Characteristics
Vav = Q / Ac;               % Vav is the average velocity (m/s) calculated
                            % from the given flow rate
Dh = 2 * h;                 % Dh is the hydraulic diameter of the channel, 
                            % equal to twice the channel height
Vc = 1.5*Vav;               % Vc is the centerline vel. of the fluid (m/s)


% Assign interpolation method
if (typ==1)|(typ==2)
  meth = 'linear';
elseif (typ==3)|(typ==4)
  meth = 'cubic';
elseif (typ==5)|(typ==6)
  meth = 'spline';
end

% Poiseuille force calculation
if rem(typ,2)==0
     N = length(z0);
     znew = z0(1:floor(N/2));           % converts z to half channel
     z_n = znew / h	;                   % normalized distance from substrate
     z_R = znew / R;			        % dist normalized to sphere radius
     z_R = round(z_R/1e-3)*1e-3;        % and then rounded
     z_R_pts = [0  1.1  1.25  1.5   2    3    4   8  1e5];
     z_n_pts  = [0  0.1  0.2   0.25  0.3  0.4  0.5];
     Fxpoints=[0, 0,      0,      0,      0,      0,     0;     ... 
               0, 0.5882, 1.1000, 1.3470, 1.5900, 2.060, 2.481; ...
               0, 0.5587, 1.0360, 1.2610, 1.4790, 1.892, 2.253; ...
               0, 0.5232, 0.9614, 1.1610, 1.3510, 1.698, 1.989; ...
               0, 0.4795, 0.8723, 1.0440, 1.2020, 1.476, 1.685; ...
               0, 0.4371, 0.7884, 0.9364, 1.0670, 1.278, 1.416; ...
               0, 0.4166, 0.7485, 0.8858, 1.0050, 1.188, 1.296; ...
               0, 0.3872, 0.6919, 0.8147, 0.9182, 1.067, 1.137; ...
               0, 0.3500, 0.6500, 0.7300, 0.8600, 0.950, 1.000];

     % This represents A(z,h)
     A = interp2(z_n_pts(1:7), z_R_pts(2:9), Fxpoints(2:9,1:7), z_n, z_R, meth);
     Fx0 = 6*pi*mu*R*Vc;
     Fx = Fx0*A;

     % Remove non interpolated values and extrapolate linearly
     % to zero based upon slope near z=R 
     % Uses first non-NaN pt and 5 above it for extrapolation
     if ~any(isnan(Fx))
        return
     end

     Fxsi = max(find(isnan(Fx))) + 1;
     if length(z_R)<(Fxsi+5)
       Fxsimax=length(z_R);
       p = polyfit(z_R(Fxsi:Fxsimax),Fx(Fxsi:Fxsimax),1);
     else
       p = polyfit(z_R(Fxsi:(Fxsi+5)),Fx(Fxsi:(Fxsi+5)),1);
     end
     Fx(1:(Fxsi-1)) = polyval(p,z_R(1:(Fxsi-1)));

     Fx(find(isnan(Fx))) = 0;        % Deal with values outside range
     if mod(N,2)==1
         Fx = [Fx, Fx(length(Fx)), fliplr(Fx)];
     else
         Fx = [Fx, fliplr(Fx)];
     end
     
     
else  % Shear flow calculation (Goldman, et al.)
     z_R_pts =  [1000 10.0677 3.7622 2.3524 1.5431 1.1276 1.0453 ...
                 1.005004 1.003202 1.0];
     Fxspts = [1.0 1.0587 1.1671 1.2780 1.4391 1.6160 1.6681 ...
               1.6969 1.6982 1.7005];
     z_R_pts = z_R_pts(length(z_R_pts):-1:1);
     Fxspts = Fxspts(length(Fxspts):-1:1);

     Fx0 = interp1(z_R_pts,Fxspts,z_R,meth);
     Fx = 6*pi*mu*R*(4*Vc/h).*z0.*Fx0;
     
     % Remove non interpolated values and extrapolate linearly
     % to zero based upon slope near z=R - assumes any problems are at small z
     % Uses first non-NaN pt and 5 above it for extrapolation
     if ~any(isnan(Fx))
        return
     end

     Fxsi = max(find(isnan(Fx))) + 1;
     if length(z_R)<(Fxsi+5)
       Fxsimax=length(z_R);
       p = polyfit(z_R(Fxsi:Fxsimax),Fx(Fxsi:Fxsimax),1);
     else
       p = polyfit(z_R(Fxsi:(Fxsi+5)),Fx(Fxsi:(Fxsi+5)),1);
     end
     Fx(1:(Fxsi-1)) = polyval(p,z_R(1:(Fxsi-1)));
     Fx(find(isnan(Fx))) = 0;        % Deal with values outside range

end