% FINDTRAJ   Find where particle ends up in force field
%   [STARTPT, ENDPT] = FINDTRAJ(X,Y,Z,F,START,FIGINFO,NORME,SLICEPLANE) finds out
%   where a particle whose position STARTPT is denoted by the 1-by-3 cell or
%   numeric array START ends up in the force field F.  Optionally, the
%   function will plot the trajectory atop slices of a user-specified field
%   FIELD at positions given by SLICEPLANE.
%
%   The function returns the 1x3 vector ENDPT of where the particle ends up
%   and STARTPT where it started
% 
% Inputs
%   X,Y,Z       space variables
%   F           4-D force matrix
%   START       can be either a 1x3 cell array of strings or a 1x3
%               numeric array, both denoting starting particle position
%   FIGINFO     0 (do not plot), 1 (plot on same plot), 2 (generate new plots)
%   FIELD       3-D matrix field
%   SLICEPLANE  1x3 cell array of where to draw slices of e-field for figures
%
%  If the last three variables are omitted, the program will not plot the
%  streamlines
%
% Output
%   ENDPT       1x3 vector of where particle ends up

function [endpt,s]=findtraj(x,y,z,f,start,R,varargin);

if nargin==6            % If we don't give all the variables then
    figinfo=0;          % we don't want to plot, so just fill in dummies
    ne=0;
    sp={[] [] []};
elseif nargin==9;           % assign variables
    figinfo = varargin{1};
    ne = real(varargin{2});
    sp = varargin{3};
end

if iscell(start)            % allows us to specify either a string or a number
    s = [0 0 0];            % for starting point
    s(1)=evalin('base',start{1});
    s(2)=evalin('base',start{2});
    s(3)=evalin('base',start{3});
elseif isnumeric(start)
    s = start;
else disp('START must be either numeric or cell array');
end

% first, check to make sure starting point is within region
if ~isinregion(x,y,z,s)
    limx = [min(min(min(x))) max(max(max(x)))];
    limy = [min(min(min(y))) max(max(max(y)))];
    limz = [min(min(min(z))) max(max(max(z)))];
    fprintf('\n\nSpecified particle starting point is x:%g y:%g z:%g\n',s);
    fprintf('Region extents are: x:%g to %g, y:%g to %g, z:%g to %g\n',limx,limy,limz);
    error('Particle starting point is not within region');
end

% compute streamline
 vr=stream3(x,y,z,f(:,:,:,1),f(:,:,:,2),f(:,:,:,3),s(1),s(2),s(3));  

% next check to make sure a non-trivial streamline was computed.  if it
% wasn't it is most likely because starting point was not within region,
% but this error should be caught by if statement above
if isempty(vr)
    error('Trivial streamline was computed.  Program will abort.  Error possibly due to force matrix');
end
endpt = vr{1,1}(size(vr{1,1},1),:);        % get endpoint of streamline
 
% plot if required
if figinfo==2
     figure
end
if figinfo>0
 slice(x,y,z,ne,sp{1},sp{2},sp{3})              % show e-field distribution 
 xlabel('X'); ylabel('Y');                      % label coordinates
 slines=streamline(vr);                         % plot streamline
 [spherex,spherey,spherez] = sphere;
 spherex = spherex*R; spherey = spherey*R; spherez = spherez*R;
 set(slines,'linewidth',2,'color','k');         % make it look pretty    
 set(gca,'XLim',[min(min(min(x))),max(max(max(x)))],...
     'YLim',[min(min(min(y))),max(max(max(y)))],...
     'ZLim',[min(min(min(z))),max(max(max(z)))]);
 hold on;
 axis equal;
 surf(spherex+endpt(1),spherey+endpt(2),spherez+endpt(3));
 drawnow
end
 

