%   RIGIDBOUND  Implements top and bottom rigid boundaries
% FTZ = RIGDIBOUND(FTZ,X,Y,Z,R,OFFSET)implements a rigid top and bottom so 
% that the particle cannot be found lower than it's radius 
% (or roff=R+offset, if desired) or higher than max(Z)-R.
%
% For bottom, essentially checks if FT < 0 at Z+ROFF and if it is replaces
% all points below with a calculated value to ensure that Fz=0 at Z=ROFF
%
% Does the complementary operation at the top boundary
%
%  Inputs:	FTZ		total z-directed force (3-D matrix)
%		    X,Y,Z	space meshgrids
%		    R		radius of particle
%		    OFFSET	offset of particle from boundary
%
%  Outputs:	FTZ		corrected total z-force (3-D)

function Ft3 = rigidbound(ft3,x,y,z,R,offset);


Ft3 = ft3;  			% total uncorrected force

if nargin==5			% If no offset is specified,
  roff = R;             % it is zero
else
  roff = R+offset;
end
croff = max(max(max(z))) - roff;    % roff w.r.t. the channel ceiling

% find the index corresponding right below roff
m_bot = max(find(squeeze(z(1,1,:))<=roff));
if m_bot == size(z,3)
    fprintf('Particle pushing against top!!!\n');
    m_bot = m_bot-1;
end

% find the index corresponding right above croff
m_top = min(find(squeeze(z(1,1,:))>=croff));

if (isempty(m_bot) & isempty(m_top))	% if the min z is > roff and the 
  return                                % max z is < croff, there is no
end                                     % need to run this function!

zmi_bot = z(1,1,m_bot);			% the z right below/at roff
zpl_bot = z(1,1,m_bot+1);		% the z right above 
zmi_top = z(1,1,m_top);			% the z right above/at croff
zpl_top = z(1,1,m_top-1);		% the z right below 

xm = size(ft3,1);		% # of nodes in x
ym = size(ft3,2);		% # of nodes in y
zm = size(ft3,3);       % # of nodes in z
xm2 = round(xm/2);		 
ym2 = round(ym/2);


% interpolate the total z-force to what it would be at roff and croff:
fR_bot = interp3(x,y,z,Ft3,x(:,:,1),y(:,:,1),roff*ones(xm,ym));
fR_top = interp3(x,y,z,Ft3,x(:,:,1),y(:,:,1),croff*ones(xm,ym));
% make a matrix such that all planes up to z=m are filled
% with what Ft3 would be at z=roff, and a second matrix covering all nodes
% that is zero below croff and fR_top above it.
fRmat_bot = zeros(xm,ym,m_bot);
for in=1:m_bot
  fRmat_bot(:,:,in) = fR_bot;
end
fRmat_top = zeros(xm,ym,zm);
for in=m_top:zm;
  fRmat_top(:,:,in) = fR_top;
end

% b_bot (b_top) will contain a list of wherever Ft3 goes <0 (>0) on the 
% plane z=roff (z=croff), but also indexed to find those z-columns under
% (above) those points in that plane
b_bot = find(fRmat_bot<0);	
fm_bot= 0*fRmat_bot(:);			% column vector of zeros
b_top = find(fRmat_top>0);	
fm_top= 0*fRmat_top(:);			% column vector of zeros

% this corresponds to all the <0 (>0) pts but in the plane above z=roff 
% (below z=croff).
bRplus_bot = b_bot(find(b_bot<=xm*ym))+xm*ym*m_bot;
bRplus_top = b_top(find(b_top>xm*ym*(zm-1)))-xm*ym*(zm-m_top);

% if roff is very close to a grid point
if abs(roff-zmi_bot)/roff < 1e-5
  % then we can't interpolate using the next formula, just negate the
  % value of at zplus (which corresponds to bRplus)
  % this is the same as bRplus but the plane right below z=roff
  bbelow_bot = b_bot(find(b_bot<=xm*ym*(m_bot-1)));
  fm_bot(bbelow_bot) =  repmat(-Ft3(bRplus_bot),m_bot-1,1);
else  
  % calculate what Ft should be right below roff in order
  % to give a Fz=0 isosurface at z=roff
  % fill the rest of the z-col below that point with the same value
  fm_bot(b_bot) = repmat(Ft3(bRplus_bot)/(zpl_bot-roff)*(zmi_bot-roff),m_bot,1);
end

% likewise for croff...
if abs(croff-zmi_top)/croff < 1e-5
  bbelow_top = b_top(find(b_top>xm*ym*(m_top)));
  fm_top(bbelow_top) =  repmat(-Ft3(bRplus_top),zm-m_top,1);
else
  fm_top(b_top) = repmat(Ft3(bRplus_top)/(zpl_top-croff)*(zmi_top-croff),m_bot,1);
end

Ft3(b_bot) = fm_bot(b_bot);
Ft3(b_top) = fm_top(b_top);
